"""
Refactor from https://github.com/r-boulland/Corporate-Website-Disclosure.git
Manually refactor:

Author: Qihang Zhang in 2023/02/03,National University of Singapore,
NUS Business School, Department of Accounting
"""
from . import WaybkJsonToCsvT
from . import cls_WayBack
